@@ -1,5 +1,5 @@
 ;;; Mailing, forwarding, and replying commands for VM
-;;; Copyright (C) 1989-1998 Kyle E. Jones
+;;; Copyright (C) 1989-2001 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -46,18 +46,11 @@
 		in-reply-to (and (not (equal "" in-reply-to)) in-reply-to))
 	  (and subject (stringp vm-reply-subject-prefix)
 	       (let ((case-fold-search t))
-		 (and
 		  (not
 		   (equal
 		    (string-match (regexp-quote vm-reply-subject-prefix)
 				  subject)
-		    0))
-		  (or
-		   (not (stringp vm-subject-ignored-prefix))
-		   (not
-		    (equal
-		     (string-match vm-subject-ignored-prefix subject)
-		     0)))))
+		    0)))
 	       (setq subject (concat vm-reply-subject-prefix subject))))
 	 (t (cond ((setq tmp (vm-get-header-contents (car mp) "Reply-To:"
 						     ", "))
@@ -299,6 +292,12 @@ vm-included-text-prefix is prepended to every yanked line."
 	      (vm-insert-region-from-buffer (vm-buffer-of message)
 					    (vm-headers-of message)
 					    (vm-text-of message))
+	      ;; decode MIME encoded words so supercite and other
+	      ;; mail-citation-hook denizens won't have to eat 'em.
+	      (if vm-display-using-mime
+		  (save-restriction
+		    (narrow-to-region start (point))
+		    (vm-decode-mime-encoded-words)))
 	      (cond ((vm-mime-types-match "multipart" type)
 		     (setq parts (copy-sequence (vm-mm-layout-parts o))))
 		    (t (setq parts (list o))))
@@ -335,10 +334,16 @@ vm-included-text-prefix is prepended to every yanked line."
 	  (set-buffer (vm-buffer-of message))
 	  (save-restriction
 	    (widen)
+	    ;; decode MIME encoded words so supercite and other
+	    ;; mail-citation-hook denizens won't have to eat 'em.
 	    (append-to-buffer b (vm-headers-of message)
 			      (vm-text-end-of message))
-	    (setq end (vm-marker (+ start (- (vm-text-end-of message)
-					     (vm-headers-of message))) b)))))
+	    (set-buffer b)
+	    (setq end (point-marker))
+	    (if vm-display-using-mime
+		(progn
+		  (narrow-to-region start end)
+		  (vm-decode-mime-encoded-words))))))
       ;; get rid of read-only text properties on the text, as
       ;; they will only cause trouble.
       (let ((inhibit-read-only t))
@@ -354,6 +359,10 @@ vm-included-text-prefix is prepended to every yanked line."
 as having been replied to, if appropriate."
   (interactive "P")
   (vm-check-for-killed-folder)
+  (if (and (boundp 'mail-alias-file)
+	   mail-alias-file
+	   (not (eq (user-uid) 0)))
+      (error "Must be superuser to use mail-alias-file.  Please set mail-alias-file to nil."))
   (let ((b (current-buffer)))
     (vm-mail-send)
     (cond ((null (buffer-name b)) ;; dead buffer
@@ -371,24 +380,7 @@ as having been replied to, if appropriate."
 	   (vm-bury-buffer b)))))
 
 (defun vm-keep-mail-buffer (buffer)
-  ;; keep this buffer if the user demands it
-  (if (memq buffer vm-kept-mail-buffers)
-      (setq vm-kept-mail-buffers
-	    (delq buffer vm-kept-mail-buffers)))
-  (setq vm-kept-mail-buffers (cons buffer vm-kept-mail-buffers)
-	vm-kept-mail-buffers (vm-delete 'buffer-name
-					vm-kept-mail-buffers t))
-  (if (not (eq vm-keep-sent-messages t))
-      (let ((extras (nthcdr (or vm-keep-sent-messages 0)
-				vm-kept-mail-buffers)))
-	(mapcar (function
-		 (lambda (b)
-		   (and (buffer-name b)
-			(not (buffer-modified-p b))
-			(kill-buffer b))))
-		extras)
-	(and vm-kept-mail-buffers extras
-	     (setcdr (memq (car extras) vm-kept-mail-buffers) nil)))))
+  (vm-keep-some-buffers buffer 'vm-kept-mail-buffers vm-keep-sent-messages))
 
 (defun vm-help-tale ()
   (save-excursion
@@ -408,15 +400,7 @@ as having been replied to, if appropriate."
       nil
     (save-restriction
       (save-excursion
-	(let ((resent nil)
-	      hostname
-	      (time (current-time)))
-	  (setq hostname (cond ((string-match "\\." (system-name))
-				(system-name))
-			       ((and (stringp 'mail-host-address)
-				     (string-match "\\." mail-host-address))
-				mail-host-address)
-			       (t "gargle.gargle.HOWL")))
+	(let ((resent nil))
 	  (if (or (vm-mail-mode-get-header-contents "Resent-To:")
 		  (vm-mail-mode-get-header-contents "Resent-Cc:")
 		  (vm-mail-mode-get-header-contents "Resent-Bcc:"))
@@ -426,11 +410,9 @@ as having been replied to, if appropriate."
 	    (vm-mail-mode-remove-header "Message-ID:"))
 	  (widen)
 	  (goto-char (point-min))
-	  (insert (format "%sMessage-ID: <%d.%d.%d.%d@%s>\n"
+	  (insert (format "%sMessage-ID: %s\n"
 			  (if resent "Resent-" "")
-			  (car time) (nth 1 time) (nth 2 time)
-			  (random 1000000)
-			  (system-name))))))))
+			  (vm-make-message-id))))))))
 
 (defun vm-mail-mode-insert-date-maybe ()
   (if (not vm-mail-header-insert-date)
@@ -452,18 +434,28 @@ as having been replied to, if appropriate."
 	  (widen)
 	  (goto-char (point-min))
 	  (insert (format "%sDate: " (if resent "Resent-" ""))
-		  (format-time-string "%a, " time)
+		  (capitalize
+		   (car (nth (string-to-int (format-time-string "%w" time))
+			     vm-weekday-alist)))
+		  ", "
 		  ;; %e generated " 2".  Go from string to int
 		  ;; to string to get rid of the blank.
 		  (int-to-string
 		   (string-to-int
 		    (format-time-string "%e" time)))
-		  (format-time-string " %b %Y %H:%M:%S" time)
+		  " "
+		  (capitalize
+		   (car (nth
+			 (1- (string-to-int (format-time-string "%m" time)))
+			 vm-month-alist)))
+		  (format-time-string " %Y %H:%M:%S" time)
 		  (format " %s%02d%02d"
 			  (if (< timezone 0) "-" "+")
 			  (abs hour)
 			  (abs min))
-		  (format-time-string " (%Z)" time)
+;; localization in Europe and elsewhere can cause %Z to return
+;; 8-bit chars, which are forbidden in headers.
+;;		  (format-time-string " (%Z)" time)
 		  "\n"))))))
 
 (defun vm-mail-mode-remove-message-id-maybe ()
@@ -488,6 +480,17 @@ as having been replied to, if appropriate."
 	      (setq resent t))
 	  (vm-mail-mode-remove-header "Date:")))))
 
+(defvar vm-dont-ask-coding-system-question nil)
+
+(cond ((and vm-fsfemacs-mule-p
+	    (not (fboundp 'vm-old-select-message-coding-system)))
+       (fset 'vm-old-select-message-coding-system
+	     (symbol-function 'select-message-coding-system))
+       (defun select-message-coding-system (&rest ignored)
+	 (if vm-dont-ask-coding-system-question
+	     nil
+	   (apply 'vm-old-select-message-coding-system ignored)))))
+
 (defun vm-mail-send ()
   "Just like mail-send except that VM flags the appropriate message(s)
 as replied to, forwarded, etc, if appropriate."
@@ -499,6 +502,7 @@ as replied to, forwarded, etc, if appropriate."
     (if (and vm-confirm-mail-send
 	     (not (y-or-n-p "Send the message? ")))
 	(error "Message not sent.")))
+  (save-excursion (run-hooks 'vm-mail-send-hook))
   (vm-mail-mode-insert-date-maybe)
   (vm-mail-mode-insert-message-id-maybe)
   ;; send mail using MIME if user requests it and if the buffer
@@ -535,7 +539,11 @@ as replied to, forwarded, etc, if appropriate."
       ;; save-excursion to be sure.
       ;;
       ;; also protect value of this-command from minibuffer reads
-      (let ((this-command this-command))
+      (let ((this-command this-command)
+	    ;; For Emacs 21.
+	    (mail-send-nonascii t)
+	    (sendmail-coding-system (vm-binary-coding-system))
+	    (vm-dont-ask-coding-system-question t))
 	(save-excursion
 	  (mail-send))))
     ;; be careful, something could have killed the composition
@@ -760,7 +768,7 @@ Subject: header manually."
 	(if miming
 	    (progn
 	      (setq mail-buffer (current-buffer))
-	      (set-buffer (generate-new-buffer "*vm-forward-buffer*"))
+	      (set-buffer (vm-make-work-buffer "*vm-forward-buffer*"))
 	      (setq header-end (point))
 	      (insert "\n"))
 	  (goto-char (point-min))
@@ -781,7 +789,11 @@ Subject: header manually."
 			(point)
 			(point-max))
 		       "\n")
-	       (insert "Content-Description: forwarded message\n"))
+	       (insert "Content-Description: forwarded message\n")
+	       ;; eight bit chars will get \201 prepended if we
+	       ;; don't do this.
+	       (if vm-fsfemacs-mule-p
+		   (set-buffer-multibyte t)))
 	      ((equal vm-forwarding-digest-type "rfc934")
 	       (vm-rfc934-encapsulate-messages
 		vm-forward-list vm-forwarded-headers
@@ -970,7 +982,7 @@ only marked messages will be put into the digest."
       (if miming
 	  (progn
 	    (setq mail-buffer (current-buffer))
-	    (set-buffer (generate-new-buffer "*vm-digest-buffer*"))
+	    (set-buffer (vm-make-work-buffer "*vm-digest-buffer*"))
 	    (setq header-end (point))
 	    (insert "\n")
 	    (setq start (point-marker)))
@@ -1094,11 +1106,47 @@ found, the current buffer remains selected."
 		      '(vm-continue-composing-message composing-message)))
       (message "No composition buffers found"))))
 
+;;;###autoload
 (defun vm-mail-to-mailto-url (url)
-  (let ((address (car (vm-parse url "^mailto:\\(.+\\)"))))
-    (vm-select-folder-buffer)
-    (vm-check-for-killed-summary)
-    (vm-mail-internal nil address)
+  (vm-session-initialization)
+  (vm-check-for-killed-folder)
+  (vm-select-folder-buffer-if-possible)
+  (vm-check-for-killed-summary)
+  (let ((list (vm-parse url "^mailto:\\([^?]+\\)\\??\\|\\([^&]+\\)&?"
+			'(1 2)))
+	to subject in-reply-to cc references newsgroups body
+	tem header value header-list)
+    (setq to (car list)
+	  to (vm-url-decode-string to)
+	  list (cdr list))
+    (while list
+      (setq tem (vm-parse (car list) "\\([^=]+\\)=?"))
+      (if (null (nth 1 tem))
+	  nil
+	(setq header (downcase (vm-url-decode-string (car tem)))
+	      value (vm-url-decode-string (nth 1 tem)))
+	(if (member header '("subject" "in-reply-to" "cc"
+			     "references" "newsgroups" "body"))
+	    ;; set the variable let-bound above
+	    (set (intern header) value)
+	  ;; we'll insert the header later
+	  (setq header-list (cons header (cons value header-list)))))
+      (setq list (cdr list)))
+    (vm-mail-internal nil to subject in-reply-to cc references newsgroups)
+    (save-excursion
+      (goto-char (point-min))
+      (while header-list
+	(insert (car header-list) ": ")
+	(capitalize-region (point) (save-excursion (beginning-of-line) (point)))
+	(insert (nth 1 header-list) "\n")
+	(setq header-list (nthcdr 2 header-list)))
+      (if (null body)
+	  nil
+	(mail-text)
+	(save-excursion (insert (vm-url-decode-string body) "\n"))
+	;; CRLF to LF for line breaks in the body
+	(while (search-forward "\r\n" nil t)
+	  (replace-match "\n"))))
     (run-hooks 'vm-mail-hook)
     (run-hooks 'vm-mail-mode-hook)))
 
@@ -1181,16 +1229,26 @@ found, the current buffer remains selected."
     (if (not (= (preceding-char) ?\n))
 	(insert ?\n))
     (insert mail-header-separator "\n")
-    (cond ((stringp mail-signature)
-	   (save-excursion
-	     (insert mail-signature)))
-	  ((eq mail-signature t)
-	   (save-excursion
-	     (insert "\n-- \n")
-	     (insert-file-contents (or (and (boundp 'mail-signature-file)
-					    (stringp mail-signature-file)
-					    mail-signature-file)
-				       "~/.signature")))))
+    (if mail-signature
+	(save-excursion
+	  (save-restriction
+	    (narrow-to-region (point) (point))
+	    (cond ((stringp mail-signature)
+		   (insert mail-signature))
+		  ((eq mail-signature t)
+		   (insert-file-contents (or (and (boundp 'mail-signature-file)
+						  (stringp mail-signature-file)
+						  mail-signature-file)
+					     "~/.signature")))
+		  (t
+		   (let ((str (eval mail-signature))) 
+		     (if (stringp str)
+			 (insert str)))))
+	    (goto-char (point-min))
+	    (if (looking-at "\n*-- \n")
+		nil
+	      (insert "\n-- \n"))
+	    (goto-char (point-max)))))
     ;; move this buffer to the head of the buffer list so window
     ;; config stuff will select it as the composition buffer.
     (vm-unbury-buffer (current-buffer))
@@ -1232,6 +1290,7 @@ found, the current buffer remains selected."
 		(list this-command 'composing-message))
     (if (null to)
 	(mail-position-on-field "To"))
+    (add-hook 'post-command-hook 'vm-update-composition-buffer-name t)
     (run-hooks 'mail-setup-hook)))
 
 (defun vm-reply-other-frame (count)
@@ -1393,7 +1452,8 @@ message."
 	  (or (vm-mail-mode-get-header-contents "From")
 	      (insert "From: " (user-login-name) "\n"))
 	  (or (vm-mail-mode-get-header-contents "Message-ID")
-	      (insert "Message-ID: <fake@fake.fake>\n"))
+	      (insert (format "Message-ID: <fake.%d.%d@fake.fake>\n"
+			      (random 1000000) (random 1000000))))
 	  (or (vm-mail-mode-get-header-contents "Date")
 	      (insert "Date: "
 		      (format-time-string "%a, %d %b %Y %H%M%S %Z"
@@ -1403,10 +1463,11 @@ message."
 	       (null (vm-mail-mode-get-header-contents "MIME-Version:"))
 	       (vm-mime-encode-composition))
 	  (vm-remove-mail-mode-header-separator)
+	  (vm-munge-message-separators 'mmdf (point-min) (point-max))
 	  (goto-char (point-min))
-	  (insert (vm-leading-message-separator vm-default-From_-folder-type))
+	  (insert (vm-leading-message-separator 'mmdf))
 	  (goto-char (point-max))
-	  (insert (vm-trailing-message-separator vm-default-From_-folder-type))
+	  (insert (vm-trailing-message-separator 'mmdf))
 	  (set-buffer-modified-p nil)
 	  ;; point of no return, don't kill it if the user quits
 	  (setq temp-buffer nil)
@@ -1424,47 +1485,34 @@ message."
 		      (list this-command) '(vm-mode startup)))
       (and temp-buffer (kill-buffer temp-buffer)))))
 
+(defun vm-update-composition-buffer-name ()
+  (if (and (eq major-mode 'mail-mode)
+           (save-match-data (string-match "^\\(mail\\|reply\\) to "
+					  (buffer-name))))
+      (let ((to (mail-fetch-field "To"))
+            (cc (mail-fetch-field "Cc"))
+	    (curbufname (buffer-name))
+	    (deactivate-mark)
+	    fmt newbufname
+            (ellipsis ""))
+	(cond (vm-reply-list (setq fmt "reply to %s%s"))
+	      (t (setq fmt "mail to %s%s")))
+        (setq to (vm-parse-addresses to)
+              cc (vm-parse-addresses cc))
+        (if (or (cdr to)
+                (and (car to) (car cc)))
+            (setq ellipsis ", ..."))
+        (setq newbufname (or (car to) (car cc) "foo (?)")
+              newbufname (funcall vm-chop-full-name-function newbufname)
+              newbufname (or (car newbufname) (car (cdr newbufname)))
+              newbufname (format fmt newbufname ellipsis))
+        (if (equal newbufname curbufname)
+            nil
+          (rename-buffer newbufname t)))))
+
 (defun vm-mail-mode-remove-tm-hooks ()
-  (make-local-hook 'mail-setup-hook)
-  (remove-hook 'mail-setup-hook 'mime/decode-message-header t)
-  (remove-hook 'mail-setup-hook 'mime/editor-mode t)
-  (make-local-hook 'mail-send-hook)
+  (remove-hook 'mail-setup-hook 'turn-on-mime-edit)
+  (remove-hook 'mail-setup-hook 'mime/decode-message-header)
+  (remove-hook 'mail-setup-hook 'mime/editor-mode)
+  (remove-hook 'mail-send-hook  'mime-edit-maybe-translate)
   (remove-hook 'mail-send-hook 'mime-editor/maybe-translate))
-
-(defvar mail-send-actions)
-
-(defun vm-compose-mail (&optional to subject other-headers continue
-		        switch-function yank-action
-			send-actions)
-  (interactive)
-  (if continue
-      (vm-continue-composing-message)
-    (let ((buffer (vm-mail-internal
-		   (if to
-		       (format "message to %s" (vm-truncate-string to 20))
-		     nil)
-		   to subject)))
-      (re-search-forward (concat "^" mail-header-separator "$"))
-      (beginning-of-line)
-      (while other-headers
-	(insert (car (car other-headers)))
-	(while (eq (char-syntax (char-before (point))) ?\ )
-	  (delete-char -1))
-	(while (eq (char-before (point)) ?:)
-	  (delete-char -1))
-	(insert ": " (cdr (car other-headers)))
-	(if (not (eq (char-before (point)) ?\n))
-	    (insert "\n"))
-	(setq other-headers (cdr other-headers)))
-      (cond ((null to)
-	     (mail-position-on-field "To"))
-	    ((null subject)
-	     (mail-position-on-field "Subject"))
-	    (t
-	     (mail-text)))
-      (funcall (or switch-function (function switch-to-buffer))
-	       (current-buffer))
-      (if yank-action
-	  (funcall (car yank-action) (cdr yank-action)))
-      (make-local-variable 'mail-send-actions)
-      (setq mail-send-actions send-actions))))
