@@ -17,8 +17,10 @@
 
 (provide 'vm-startup)
 
+(defvar enable-multibyte-characters)
+
 ;;;###autoload
-(defun vm (&optional folder read-only)
+(defun vm (&optional folder read-only access-method)
   "Read mail under Emacs.
 Optional first arg FOLDER specifies the folder to visit.  It defaults
 to the value of vm-primary-inbox.  The folder buffer is put into VM
@@ -35,8 +37,8 @@ be moved and appended to the resulting buffer.
 All the messages can be read by repeatedly pressing SPC.  Use `n'ext and
 `p'revious to move about in the folder.  Messages are marked for
 deletion with `d', and saved to another folder with `s'.  Quitting VM
-with `q' expunges deleted messages and saves the buffered folder to
-disk.
+with `q' saves the buffered folder to disk, but does not expunge
+deleted messages.  Use `###' to expunge deleted messages.
 
 See the documentation for vm-mode for more information."
   (interactive (list nil current-prefix-arg))
@@ -48,7 +50,14 @@ See the documentation for vm-mode for more information."
     (let ((full-startup (not (bufferp folder)))
 	  (did-read-index-file nil)
 	  folder-buffer first-time totals-blurb
+	  folder-name remote-spec
 	  preserve-auto-save-file)
+      (cond ((eq access-method 'pop)
+	     (setq remote-spec (vm-pop-find-spec-for-name folder))
+	     (if (null remote-spec)
+		 (error "No such POP folder: %s" folder))
+	     (setq folder-name folder
+		   folder (vm-pop-make-filename-for-spec remote-spec))))
       (setq folder-buffer
 	    (if (bufferp folder)
 		folder
@@ -64,26 +73,39 @@ See the documentation for vm-mode for more information."
 				  default-directory))
 			    (inhibit-local-variables t)
 			    (enable-local-variables nil)
+			    (enable-local-eval nil)
+			    ;; for Emacs/MULE
+			    (default-enable-multibyte-characters nil)
 			    ;; for XEmacs/Mule
-			    (coding-system-for-read 'no-conversion))
+			    (coding-system-for-read
+			         (vm-line-ending-coding-system)))
 			(message "Reading %s..." file)
 			(prog1 (find-file-noselect file)
 			  ;; update folder history
-			  (let ((item (or folder vm-primary-inbox)))
+			  (let ((item (or remote-spec folder
+					  vm-primary-inbox)))
 			    (if (not (equal item (car vm-folder-history)))
 				(setq vm-folder-history
 				      (cons item vm-folder-history))))
 			  (message "Reading %s... done" file))))))))
       (set-buffer folder-buffer)
-      ;; for XEmacs/MULE
+      (cond ((eq access-method 'pop)
+	     (if (not (equal folder-name (buffer-name)))
+		 (rename-buffer folder-name t))))
+      (if (and vm-fsfemacs-mule-p enable-multibyte-characters)
+	  (set-buffer-multibyte nil))
+      ;; for MULE
       ;;
       ;; If the file coding system is not a no-conversion variant,
-      ;; make it so by encoding all the text, then setting
-      ;; the file coding system and decoding it.
-      ;; This is only possible if a file is visited and then vm-mode
-      ;; is run on it afterwards.
+      ;; make it so by encoding all the text, then setting the
+      ;; file coding system and decoding it.  This situation is
+      ;; only possible if a file is visited and then vm-mode is
+      ;; run on it afterwards.
+      ;;
+      ;; There are separate code blocks for FSF Emacs and XEmacs
+      ;; because the coding systems have different names.
       (defvar buffer-file-coding-system)
-      (if (and vm-xemacs-mule-p
+      (if (and (or vm-xemacs-mule-p vm-xemacs-file-coding-p)
 	       (not (eq (get-coding-system buffer-file-coding-system)
 			(get-coding-system 'no-conversion-unix)))
 	       (not (eq (get-coding-system buffer-file-coding-system)
@@ -102,6 +124,27 @@ See the documentation for vm-mode for more information."
 		  (decode-coding-region (point-min) (point-max)
 					buffer-file-coding-system))
 	      (set-buffer-modified-p omodified))))
+      (if (and vm-fsfemacs-mule-p (null buffer-file-coding-system))
+	  (set-buffer-file-coding-system 'raw-text nil))
+      (if (and vm-fsfemacs-mule-p
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'raw-text-unix)))
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'raw-text-mac)))
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'raw-text-dos)))
+	       (not (eq (coding-system-base buffer-file-coding-system)
+			(coding-system-base 'no-conversion))))
+	  (let ((buffer-read-only nil)
+		(omodified (buffer-modified-p)))
+	    (unwind-protect
+		(progn
+		  (encode-coding-region (point-min) (point-max)
+					buffer-file-coding-system)
+		  (set-buffer-file-coding-system 'raw-text nil)
+		  (decode-coding-region (point-min) (point-max)
+					buffer-file-coding-system))
+	      (set-buffer-modified-p omodified))))
       (vm-check-for-killed-summary)
       (vm-check-for-killed-presentation)
       ;; If the buffer's not modified then we know that there can be no
@@ -134,7 +177,14 @@ See the documentation for vm-mode for more information."
 	    (buffer-disable-undo (current-buffer))
 	    (abbrev-mode 0)
 	    (auto-fill-mode 0)
-	    (vm-mode-internal)
+	    ;; If an 8-bit message arrives undeclared the 8-bit
+	    ;; characters in it should be displayed using the
+	    ;; user's default face charset, rather than as octal
+	    ;; escapes.
+	    (vm-fsfemacs-nonmule-display-8bit-chars)
+	    (vm-mode-internal access-method)
+	    (cond ((eq access-method 'pop)
+		   (vm-set-folder-pop-maildrop-spec remote-spec)))
 	    ;; If the buffer is modified we don't know if the
 	    ;; folder format has been changed to be different
 	    ;; from index file, so don't read the index file in
@@ -142,13 +192,16 @@ See the documentation for vm-mode for more information."
 	    (if (not (buffer-modified-p))
 		(setq did-read-index-file (vm-read-index-file-maybe)))))
 
-      (vm-assimilate-new-messages nil (not did-read-index-file) nil)
+      ;; builds message list, reads attributes if they weren't
+      ;; read from an index file.
+      (vm-assimilate-new-messages nil (not did-read-index-file) nil t)
 
       (if (and first-time (not did-read-index-file))
 	  (progn
 	    (vm-gobble-visible-header-variables)
 	    (vm-gobble-bookmark)
 	    (vm-gobble-pop-retrieved)
+	    (vm-gobble-imap-retrieved)
 	    (vm-gobble-summary)
 	    (vm-gobble-labels)))
 
@@ -178,19 +231,17 @@ See the documentation for vm-mode for more information."
       ;; say this NOW, before the non-previewers read a message,
       ;; alter the new message count and confuse themselves.
       (if full-startup
-	  ;; save blurb so we can repeat it later as necessary.
-	  (setq totals-blurb (vm-emit-totals-blurb)))
+	  (progn
+	    ;; save blurb so we can repeat it later as necessary.
+	    (setq totals-blurb (vm-emit-totals-blurb))
+	    (and buffer-file-name
+		 (vm-store-folder-totals buffer-file-name (cdr vm-totals)))))
 
       (vm-thoughtfully-select-message)
       (vm-update-summary-and-mode-line)
       ;; need to do this after any frame creation because the
       ;; toolbar sets frame-specific height and width specifiers.
-      (and (vm-toolbar-support-possible-p) vm-use-toolbar
-	   (progn
-	     (message "Initializing toolbar...")
-	     (vm-toolbar-install-toolbar)
-	     (message "Initializing toolbar... done")
-	     (vm-toolbar-update-toolbar)))
+      (vm-toolbar-install-or-uninstall-toolbar)
 
       (and vm-use-menus (vm-menu-support-possible-p)
 	   (vm-menu-install-visited-folders-menu))
@@ -200,7 +251,7 @@ See the documentation for vm-mode for more information."
 	    (if (and (vm-should-generate-summary)
 		     ;; don't generate a summary if recover-file is
 		     ;; likely to happen, since recover-file does
-		     ;; nothing useful in a summary buffer.
+		     ;; not work in a summary buffer.
 		     (not preserve-auto-save-file))
 		(vm-summarize t nil))
 	    ;; raise the summary frame if the user wants frames
@@ -222,12 +273,16 @@ See the documentation for vm-mode for more information."
 				      (current-buffer)))))))
 
       (if vm-message-list
-	  (vm-preview-current-message))
+	  ;; don't decode MIME if recover-file is
+	  ;; likely to happen, since recover-file does
+	  ;; not work in a presentation buffer.
+	  (let ((vm-auto-decode-mime-messages
+		 (and vm-auto-decode-mime-messages
+		      (not preserve-auto-save-file))))
+	    (vm-preview-current-message)))
 
       (run-hooks 'vm-visit-folder-hook)
 
-      (if full-startup
-	  (message totals-blurb))
       ;; Warn user about auto save file, if appropriate.
       (if (and full-startup preserve-auto-save-file)
 	  (message 
@@ -238,13 +293,17 @@ See the documentation for vm-mode for more information."
       ;; stop here.
       (if (or (not full-startup) preserve-auto-save-file)
 	  (throw 'done t))
+      
+      (if full-startup
+	  (message totals-blurb))
+
       (if (and vm-auto-get-new-mail
 	       (not vm-block-new-mail)
 	       (not vm-folder-read-only))
 	  (progn
 	    (message "Checking for new mail for %s..."
-				(or buffer-file-name (buffer-name)))
-	    (if (and (vm-get-spooled-mail t) (vm-assimilate-new-messages t))
+		     (or buffer-file-name (buffer-name)))
+	    (if (vm-get-spooled-mail t)
 		(progn
 		  (setq totals-blurb (vm-emit-totals-blurb))
 		  (if (vm-thoughtfully-select-message)
@@ -293,10 +352,11 @@ See the documentation for vm-mode for more information."
 (defun vm-mode (&optional read-only)
   "Major mode for reading mail.
 
-This is VM 6.62.
+This is VM 7.03.
 
 Commands:
    h - summarize folder contents
+   H - summarize contents of all folders
  C-t - toggle threads display
 
    n - go to next message
@@ -316,11 +376,16 @@ Commands:
    b - scroll backward a page
    < - go to beginning of current message
    > - go to end of current message
+   [ - go to previous button
+   ] - go to next button
+   D - decode MIME if not already decoded.  If already decoded,
+       display all MIME objects as tags.  If already displaying
+       tags, show raw unecoded MIME>
 
    d - delete message, prefix arg deletes messages forward
  C-d - delete message, prefix arg deletes messages backward
    u - undelete
-   k - flag for deletion all messages with same subject as the current message
+   k - delete all messages with same subject as the current message
 
    r - reply (only to the sender of the message)
    R - reply with included text from the current message
@@ -350,7 +415,7 @@ Commands:
    w - write current message to a file without its headers (appends if exists)
    S - save entire folder to disk, does not expunge
    A - save unfiled messages to their vm-auto-folder-alist specified folders
-   # - expunge deleted messages (without saving folder)
+ ### - expunge deleted messages (without saving folder)
    q - quit VM, deleted messages are not expunged, folder is
        saved to disk if it is modified.  new messages are changed
        to be flagged as just unread.
@@ -375,7 +440,12 @@ Commands:
        M R - mark messages within the point/mark region in the summary
        M r - unmark messages within the point/mark region in the summary
        M V - toggle the marked-ness of all messages
-
+       M X - apply the selectors of a named virtual folder to the
+             messages in the current folder and mark all messages
+             that match those selectors.
+       M x - apply the selectors of a named virtual folder to the
+             messages in the current folder and unmark all messages
+             that match those selectors.
        M ? - partial help for mark commands
 
  W S - save the current window configuration to a name
@@ -407,6 +477,16 @@ Commands:
  l a - add labels to message
  l d - delete labels from message
 
+   $ - prefix for MIME commands.  Position the cursor over a MIME
+       tag and use these keystrokes to operate on a MIME object.
+
+       $ s - save the MIME object
+       $ p - print the MIME object
+       $ | - pipe the MIME object to a shell command.
+       $ RET - display the MIME object's text using the \"default\" face.
+       $ e - display the MIME object with an external viewer.
+       $ d - delete the MIME object from the message.
+
    L - reload your VM init file, ~/.vm
 
    % - change a folder to another type
@@ -454,8 +534,13 @@ Variables:
    vm-display-buffer-hook
    vm-display-using-mime
    vm-edit-message-hook
+   vm-fill-paragraphs-containing-long-lines
+   vm-flush-interval
    vm-folder-directory
    vm-folder-read-only
+   vm-folders-summary-database
+   vm-folders-summary-directories
+   vm-folders-summary-format
    vm-follow-summary-cursor
    vm-forward-message-hook
    vm-forwarded-headers
@@ -466,17 +551,21 @@ Variables:
    vm-frame-per-composition
    vm-frame-per-edit
    vm-frame-per-folder
+   vm-frame-per-folders-summary
    vm-frame-per-help
    vm-frame-per-summary
    vm-highlighted-header-face
    vm-highlighted-header-regexp
    vm-honor-page-delimiters
    vm-image-directory
+   vm-imagemagick-convert-program
+   vm-imagemagick-identify-program
    vm-imap-auto-expunge-alist
    vm-imap-bytes-per-session
    vm-imap-expunge-after-retrieving
    vm-imap-max-message-size
    vm-imap-messages-per-session
+   vm-imap-session-preauth-hook
    vm-index-file-suffix
    vm-in-reply-to-format
    vm-included-text-attribution-format
@@ -489,11 +578,13 @@ Variables:
    vm-jump-to-unread-messages
    vm-keep-crash-boxes
    vm-keep-sent-messages
+   vm-lynx-program
    vm-mail-check-interval
    vm-mail-header-from
    vm-mail-header-insert-date
    vm-mail-header-insert-message-id
    vm-mail-mode-hook
+   vm-mail-send-hook
    vm-make-crash-box-name
    vm-make-spool-file-name
    vm-mime-7bit-composition-charset
@@ -507,10 +598,16 @@ Variables:
    vm-mime-base64-decoder-switches
    vm-mime-base64-encoder-program
    vm-mime-base64-encoder-switches
-   vm-mime-button-format-alist
    vm-mime-button-face
+   vm-mime-button-format-alist
+   vm-mime-charset-converter-alist
    vm-mime-charset-font-alist
+   vm-mime-confirm-delete
+   vm-mime-decode-for-preview
+   vm-mime-default-face-charset-exceptions
    vm-mime-default-face-charsets
+   vm-mime-delete-after-saving
+   vm-mime-delete-viewer-processes
    vm-mime-digest-discard-header-regexp
    vm-mime-digest-headers
    vm-mime-display-function
@@ -519,7 +616,15 @@ Variables:
    vm-mime-internal-content-type-exceptions
    vm-mime-internal-content-types
    vm-mime-max-message-size
+   vm-mime-qp-decoder-program
+   vm-mime-qp-decoder-switches
+   vm-mime-qp-encoder-program
+   vm-mime-qp-encoder-switches
+   vm-mime-require-mime-version-header
    vm-mime-type-converter-alist
+   vm-mime-use-image-strips
+   vm-mime-uuencode-decoder-program
+   vm-mime-uuencode-decoder-switches
    vm-mode-hook
    vm-mosaic-program
    vm-mosaic-program-switches
@@ -531,9 +636,12 @@ Variables:
    vm-mutable-windows
    vm-netscape-program
    vm-netscape-program-switches
+   vm-page-continuation-glyph
+   vm-paragraph-fill-column
    vm-pop-auto-expunge-alist
    vm-pop-bytes-per-session
    vm-pop-expunge-after-retrieving
+   vm-pop-folder-alist
    vm-pop-max-message-size
    vm-pop-md5-program
    vm-pop-messages-per-session
@@ -570,8 +678,16 @@ Variables:
    vm-skip-read-messages
    vm-spool-file-suffixes
    vm-spool-files
+   vm-spooled-mail-waiting-hook
+   vm-ssh-program
+   vm-ssh-program-switches
+   vm-ssh-remote-command
    vm-startup-with-summary
    vm-strip-reply-headers
+   vm-stunnel-program
+   vm-stunnel-program-switches
+   vm-stunnel-random-data-method
+   vm-subject-significant-chars
    vm-summary-arrow
    vm-summary-format
    vm-summary-highlight-face
@@ -587,6 +703,7 @@ Variables:
    vm-undisplay-buffer-hook
    vm-unforwarded-header-regexp
    vm-url-browser
+   vm-url-retrieval-methods
    vm-url-search-limit
    vm-use-menus
    vm-use-toolbar
@@ -596,6 +713,7 @@ Variables:
    vm-visit-folder-hook
    vm-visit-when-saving
    vm-warp-mouse-to-new-frame
+   vm-wget-program
    vm-window-configuration-file
 "
   (interactive "P")
@@ -619,7 +737,7 @@ visited folder."
    (save-excursion
      (vm-session-initialization)
      (vm-check-for-killed-folder)
-     (vm-select-folder-buffer)
+     (vm-select-folder-buffer-if-possible)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
 				default-directory))
@@ -636,12 +754,19 @@ visited folder."
 	     current-prefix-arg))))
   (vm-session-initialization)
   (vm-check-for-killed-folder)
-  (vm-select-folder-buffer)
+  (vm-select-folder-buffer-if-possible)
   (vm-check-for-killed-summary)
   (setq vm-last-visit-folder folder)
-  (let ((default-directory (or vm-folder-directory default-directory)))
-    (setq folder (expand-file-name folder)))
-  (vm folder read-only))
+  (let ((access-method nil) foo)
+    (cond ((and (stringp vm-recognize-pop-maildrops)
+		(string-match vm-recognize-pop-maildrops folder)
+		(setq foo (vm-pop-find-name-for-spec folder)))
+	   (setq folder foo
+		 access-method 'pop))
+	  (t
+	   (let ((default-directory (or vm-folder-directory default-directory)))
+	     (setq folder (expand-file-name folder)))))
+    (vm folder read-only access-method)))
 
 ;;;###autoload
 (defun vm-visit-folder-other-frame (folder &optional read-only)
@@ -650,7 +775,7 @@ visited folder."
    (save-excursion
      (vm-session-initialization)
      (vm-check-for-killed-folder)
-     (vm-select-folder-buffer)
+     (vm-select-folder-buffer-if-possible)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
 				default-directory))
@@ -681,7 +806,7 @@ visited folder."
    (save-excursion
      (vm-session-initialization)
      (vm-check-for-killed-folder)
-     (vm-select-folder-buffer)
+     (vm-select-folder-buffer-if-possible)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
 				default-directory))
@@ -704,6 +829,108 @@ visited folder."
 	(vm-search-other-frames nil))
     (vm-visit-folder folder read-only)))
 
+;;;###autoload
+(defun vm-visit-pop-folder (folder &optional read-only)
+  "Visit a POP mailbox.
+VM will parse and present its messages to you in the usual way.
+
+First arg FOLDER specifies the name of the POP mailbox to visit.
+You can only visit mailboxes that are specified in `vm-pop-folder-alist'.
+When this command is called interactively the mailbox name is read from the
+minibuffer.
+
+Prefix arg or optional second arg READ-ONLY non-nil indicates
+that the folder should be considered read only.  No attribute
+changes, messages additions or deletions will be allowed in the
+visited folder."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((completion-list (mapcar (function (lambda (x) (nth 1 x)))
+				    vm-pop-folder-alist))
+	   (default vm-last-visit-pop-folder)
+	   (this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-string
+	      (format "Visit%s POP folder:%s "
+		      (if current-prefix-arg " read only" "")
+		      (if default
+			  (format " (default %s)" default)
+			""))
+	      completion-list)
+	     current-prefix-arg))))
+  (vm-session-initialization)
+  (vm-check-for-killed-folder)
+  (vm-select-folder-buffer-if-possible)
+  (vm-check-for-killed-summary)
+  (if (and (equal folder "") (stringp vm-last-visit-pop-folder))
+      (setq folder vm-last-visit-pop-folder))
+  (if (null (vm-pop-find-spec-for-name folder))
+      (error "No such POP folder: %s" folder))
+  (setq vm-last-visit-pop-folder folder)
+  (vm folder read-only 'pop))
+
+;;;###autoload
+(defun vm-visit-pop-folder-other-frame (folder &optional read-only)
+  "Like vm-visit-pop-folder, but run in a newly created frame."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((completion-list (mapcar (function (lambda (x) (nth 1 x)))
+				    vm-pop-folder-alist))
+	   (default vm-last-visit-pop-folder)
+	   (this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-string
+	      (format "Visit%s POP folder:%s "
+		      (if current-prefix-arg " read only" "")
+		      (if default
+			  (format " (default %s)" default)
+			""))
+	      completion-list)
+	     current-prefix-arg))))
+  (vm-session-initialization)
+  (if (vm-multiple-frames-possible-p)
+      (vm-goto-new-frame 'folder))
+  (let ((vm-frame-per-folder nil)
+	(vm-search-other-frames nil))
+    (vm-visit-pop-folder folder read-only))
+  (if (vm-multiple-frames-possible-p)
+      (vm-set-hooks-for-frame-deletion)))
+
+;;;###autoload
+(defun vm-visit-pop-folder-other-window (folder &optional read-only)
+  "Like vm-visit-pop-folder, but run in a different window."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((completion-list (mapcar (function (lambda (x) (nth 1 x)))
+				    vm-pop-folder-alist))
+	   (default vm-last-visit-pop-folder)
+	   (this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-string
+	      (format "Visit%s POP folder:%s "
+		      (if current-prefix-arg " read only" "")
+		      (if default
+			  (format " (default %s)" default)
+			""))
+	      completion-list)
+	     current-prefix-arg))))
+  (vm-session-initialization)
+  (if (one-window-p t)
+      (split-window))
+  (other-window 1)
+  (let ((vm-frame-per-folder nil)
+	(vm-search-other-frames nil))
+    (vm-visit-pop-folder folder read-only)))
+
 (put 'vm-virtual-mode 'mode-class 'special)
 
 (defun vm-virtual-mode (&rest ignored)
@@ -747,6 +974,7 @@ vm-visit-virtual-folder.")
 	      (funcall x (current-buffer))))
 	  (abbrev-mode 0)
 	  (auto-fill-mode 0)
+	  (vm-fsfemacs-nonmule-display-8bit-chars)
 	  (setq mode-name "VM Virtual"
 		mode-line-format vm-mode-line-format
 		buffer-read-only t
@@ -785,8 +1013,7 @@ vm-visit-virtual-folder.")
     (if vm-raise-frame-at-startup
 	(vm-raise-frame))
     (vm-display nil nil (list this-command) (list this-command 'startup))
-    (and (vm-toolbar-support-possible-p) vm-use-toolbar
-	 (vm-toolbar-install-toolbar))
+    (vm-toolbar-install-or-uninstall-toolbar)
     (if first-time
 	(progn
 	  (if (vm-should-generate-summary)
@@ -866,7 +1093,7 @@ recipient list."
   (interactive)
   (vm-session-initialization)
   (vm-check-for-killed-folder)
-  (vm-select-folder-buffer)
+  (vm-select-folder-buffer-if-possible)
   (vm-check-for-killed-summary)
   (vm-mail-internal nil to)
   (run-hooks 'vm-mail-hook)
@@ -901,15 +1128,133 @@ recipient list."
 	(vm-search-other-frames nil))
     (vm-mail to)))
 
+(fset 'vm-folders-summary-mode 'vm-mode)
+(put 'vm-folders-summary-mode 'mode-class 'special)
+
+;;;###autoload
+(defun vm-folders-summarize (&optional display raise)
+  "Generate a summary of the folders in your folder directories.
+Set `vm-folders-summary-directories' to specify the folder directories.
+Press RETURN or click mouse button 2 on an entry in the folders
+summary buffer to select a folder."
+  (interactive "p\np")
+  (vm-session-initialization)
+  (vm-check-for-killed-summary)
+  (if (not (featurep 'berkeley-db))
+      (error "Berkeley DB support needed to run this command"))
+  (if (null vm-folders-summary-database)
+      (error "'vm-folders-summary-database' must be non-nil to run this command"))
+  (if (null vm-folders-summary-buffer)
+      (let ((folder-buffer (and (eq major-mode 'vm-mode)
+				(current-buffer))))
+	(setq vm-folders-summary-buffer
+	      (let ((default-enable-multibyte-characters t))
+		(get-buffer-create "VM Folders Summary")))
+	(save-excursion
+	  (set-buffer vm-folders-summary-buffer)
+	  (abbrev-mode 0)
+	  (auto-fill-mode 0)
+	  (vm-fsfemacs-nonmule-display-8bit-chars)
+	  (if (fboundp 'buffer-disable-undo)
+	      (buffer-disable-undo (current-buffer))
+	    ;; obfuscation to make the v19 compiler not whine
+	    ;; about obsolete functions.
+	    (let ((x 'buffer-flush-undo))
+	      (funcall x (current-buffer))))
+	  (vm-folders-summary-mode-internal))
+	(vm-make-folders-summary-associative-hashes)
+	(vm-do-folders-summary)))
+  ;; if this command was run from a VM related buffer, select
+  ;; the folder buffer in the folders summary, but only if that
+  ;; folder has an entry there.
+  (and vm-mail-buffer
+       (vm-check-for-killed-folder))
+  (save-excursion
+    (and vm-mail-buffer
+	 (vm-select-folder-buffer))
+    (vm-check-for-killed-summary)
+    (let ((folder-buffer (and (eq major-mode 'vm-mode)
+			      (current-buffer)))
+	  fs )
+      (if (or (null vm-folders-summary-hash) (null folder-buffer)
+	      (null buffer-file-name))
+	  nil
+	(setq fs (symbol-value (intern-soft (vm-make-folders-summary-key
+					     buffer-file-name)
+					    vm-folders-summary-hash)))
+	(if (null fs)
+	    nil
+	  (vm-mark-for-folders-summary-update buffer-file-name)
+	  (set-buffer vm-folders-summary-buffer)
+	  (setq vm-mail-buffer folder-buffer)))))
+  (if display
+      (save-excursion
+	(vm-goto-new-folders-summary-frame-maybe)
+	(vm-display vm-folders-summary-buffer t
+		    '(vm-folders-summarize)
+		    (list this-command) (not raise))
+	;; need to do this after any frame creation because the
+	;; toolbar sets frame-specific height and width specifiers.
+	(set-buffer vm-folders-summary-buffer)
+	(vm-toolbar-install-or-uninstall-toolbar))
+    (vm-display nil nil '(vm-folders-summarize)
+		(list this-command)))
+  (vm-update-summary-and-mode-line))
+
+(defvar mail-send-actions)
+
+;;;###autoload
+(defun vm-compose-mail (&optional to subject other-headers continue
+		        switch-function yank-action
+			send-actions)
+  (interactive)
+  (vm-session-initialization)
+  (if continue
+      (vm-continue-composing-message)
+    (let ((buffer (vm-mail-internal
+		   (if to
+		       (format "message to %s"
+			       (vm-truncate-roman-string to 20))
+		     nil)
+		   to subject)))
+      (goto-char (point-min))
+      (re-search-forward (concat "^" mail-header-separator "$"))
+      (beginning-of-line)
+      (while other-headers
+	(insert (car (car other-headers)))
+	(while (eq (char-syntax (char-before (point))) ?\ )
+	  (delete-char -1))
+	(while (eq (char-before (point)) ?:)
+	  (delete-char -1))
+	(insert ": " (cdr (car other-headers)))
+	(if (not (eq (char-before (point)) ?\n))
+	    (insert "\n"))
+	(setq other-headers (cdr other-headers)))
+      (cond ((null to)
+	     (mail-position-on-field "To"))
+	    ((null subject)
+	     (mail-position-on-field "Subject"))
+	    (t
+	     (mail-text)))
+      (funcall (or switch-function (function switch-to-buffer))
+	       (current-buffer))
+      (if yank-action
+	  (apply (car yank-action) (cdr yank-action)))
+      (make-local-variable 'mail-send-actions)
+      (setq mail-send-actions send-actions))))
+
 ;;;###autoload
 (defun vm-submit-bug-report ()
   "Submit a bug report, with pertinent information to the VM bug list."
   (interactive)
   (require 'reporter)
-  (require 'vm-version)
-  (require 'vm-vars)
-  ;; make sure the user doesn't try to use vm-mail here.
-  (let ((reporter-mailer '(mail)))
+  (vm-session-initialization)
+  ;; Use VM to send the bug report.  Could be trouble if vm-mail
+  ;; is what the user wants to complain about.  But most of the
+  ;; time we'll be fine and users like to use MIME to attach
+  ;; stuff to the reports.
+  (let ((reporter-mailer '(vm-mail))
+	(mail-user-agent 'vm-user-agent))
     (delete-other-windows)
     (reporter-submit-bug-report
      vm-maintainer-address
@@ -949,9 +1294,13 @@ recipient list."
       'vm-display-using-mime
       'vm-edit-message-hook
       'vm-edit-message-mode
+      'vm-fill-paragraphs-containing-long-lines
       'vm-flush-interval
       'vm-folder-directory
       'vm-folder-read-only
+      'vm-folders-summary-database
+      'vm-folders-summary-directories
+      'vm-folders-summary-format
       'vm-follow-summary-cursor
       'vm-forward-message-hook
       'vm-forwarded-headers
@@ -962,18 +1311,22 @@ recipient list."
       'vm-frame-per-composition
       'vm-frame-per-edit
       'vm-frame-per-folder
+      'vm-frame-per-folders-summary
       'vm-frame-per-help
       'vm-frame-per-summary
       'vm-highlight-url-face
       'vm-highlighted-header-regexp
       'vm-honor-page-delimiters
       'vm-image-directory
+      'vm-imagemagick-convert-program
+      'vm-imagemagick-identify-program
 ;; IMAP passwords might be listed here
 ;;      'vm-imap-auto-expunge-alist
       'vm-imap-bytes-per-session
       'vm-imap-expunge-after-retrieving
       'vm-imap-max-message-size
       'vm-imap-messages-per-session
+      'vm-imap-session-preauth-hook
       'vm-in-reply-to-format
       'vm-included-text-attribution-format
       'vm-included-text-discard-header-regexp
@@ -987,14 +1340,16 @@ recipient list."
       'vm-jump-to-unread-messages
       'vm-keep-crash-boxes
       'vm-keep-sent-messages
+      'vm-lynx-program
       'vm-mail-header-from
       'vm-mail-header-insert-date
       'vm-mail-header-insert-message-id
       'vm-mail-hook
-      'vm-make-crash-box-name
-      'vm-make-spool-file-name
       'vm-mail-check-interval
       'vm-mail-mode-hook
+      'vm-mail-send-hook
+      'vm-make-crash-box-name
+      'vm-make-spool-file-name
       'vm-mime-7bit-composition-charset
       'vm-mime-8bit-composition-charset
       'vm-mime-8bit-text-transfer-encoding
@@ -1006,10 +1361,16 @@ recipient list."
       'vm-mime-base64-decoder-switches
       'vm-mime-base64-encoder-program
       'vm-mime-base64-encoder-switches
-      'vm-mime-button-format-alist
       'vm-mime-button-face
+      'vm-mime-button-format-alist
+      'vm-mime-charset-converter-alist
       'vm-mime-charset-font-alist
+      'vm-mime-confirm-delete
+      'vm-mime-decode-for-preview
+      'vm-mime-default-face-charset-exceptions
       'vm-mime-default-face-charsets
+      'vm-mime-delete-after-saving
+      'vm-mime-delete-viewer-processes
       'vm-mime-digest-discard-header-regexp
       'vm-mime-digest-headers
       'vm-mime-display-function
@@ -1018,7 +1379,15 @@ recipient list."
       'vm-mime-internal-content-type-exceptions
       'vm-mime-internal-content-types
       'vm-mime-max-message-size
+      'vm-mime-qp-decoder-program
+      'vm-mime-qp-decoder-switches
+      'vm-mime-qp-encoder-program
+      'vm-mime-qp-encoder-switches
+      'vm-mime-require-mime-version-header
       'vm-mime-type-converter-alist
+      'vm-mime-use-image-strips
+      'vm-mime-uuencode-decoder-program
+      'vm-mime-uuencode-decoder-switches
       'vm-mode-hook
       'vm-mode-hooks
       'vm-mosaic-program
@@ -1032,10 +1401,14 @@ recipient list."
       'vm-mutable-windows
       'vm-netscape-program
       'vm-netscape-program-switches
+      'vm-page-continuation-glyph
+      'vm-paragraph-fill-column
 ;; POP passwords might be listed here
 ;;      'vm-pop-auto-expunge-alist
       'vm-pop-bytes-per-session
       'vm-pop-expunge-after-retrieving
+;; POP passwords might be listed here
+;;      'vm-pop-folder-alist
       'vm-pop-max-message-size
       'vm-pop-messages-per-session
       'vm-pop-md5-program
@@ -1074,8 +1447,16 @@ recipient list."
 ;; don't send vm-spool-files by default, might contain passwords
 ;;      'vm-spool-files
       'vm-spool-file-suffixes
+      'vm-spooled-mail-waiting-hook
+      'vm-ssh-program
+      'vm-ssh-program-switches
+      'vm-ssh-remote-command
       'vm-startup-with-summary
       'vm-strip-reply-headers
+      'vm-stunnel-program
+      'vm-stunnel-program-switches
+      'vm-stunnel-random-data-method
+      'vm-subject-significant-chars
       'vm-summary-format
       'vm-summary-highlight-face
       'vm-summary-mode-hook
@@ -1093,6 +1474,7 @@ recipient list."
       'vm-undisplay-buffer-hook
       'vm-unforwarded-header-regexp
       'vm-url-browser
+      'vm-url-retrieval-methods
       'vm-url-search-limit
       'vm-use-menus
       'vm-use-toolbar
@@ -1102,19 +1484,20 @@ recipient list."
       'vm-visit-folder-hook
       'vm-visit-when-saving
       'vm-warp-mouse-to-new-frame
+      'vm-wget-program
       'vm-window-configuration-file
 ;; see what the user had loaded
       'features
       )
      nil
      nil
-     "Please change the Subject header to a concise bug description.\nRemember to cover the basics, that is, what you expected to\nhappen and what in fact did happen.  Please remove these\ninstructions from your message.")
+     "Please change the Subject header to a concise bug description.\nIn this report, remember to cover the basics, that is, what you expected to\nhappen and what in fact did happen.  Please remove these\ninstructions from your message.")
     (save-excursion
       (goto-char (point-min))
-      (mail-position-on-field "Subject")
-      (beginning-of-line)
-      (delete-region (point) (progn (forward-line) (point)))
-      (insert "Subject: VM " vm-version " induces a brain tumor in the user.\n         It is the tumor that creates the hallucinations.\n"))))
+      (mail-position-on-field "Subject"))))
+;;      (beginning-of-line)
+;;      (delete-region (point) (progn (forward-line) (point)))
+;;      (insert "Subject: VM " vm-version " induces a brain tumor in the user.\n         It is the tumor that creates the hallucinations.\n"))))
 
 (defun vm-load-init-file (&optional interactive)
   (interactive "p")
@@ -1138,9 +1521,7 @@ recipient list."
 		  (and (= emacs-major-version 19)
 		       (< emacs-minor-version 34))))
 	 (error "VM %s must be run on Emacs 19.34 or a later v19 version."
-		vm-version))
-	((and vm-fsfemacs-p (= emacs-major-version 20))
-	 (error "VM has not been ported to v20 Emacs.  Running VM in this environment is not advised."))))
+		vm-version))))
 
 (defun vm-set-debug-flags ()
   (or stack-trace-on-error
@@ -1159,6 +1540,7 @@ recipient list."
   (require 'vm)
   (vm-note-emacs-version)
   (vm-check-emacs-version)
+  (add-hook 'kill-emacs-hook 'vm-garbage-collect-global)
 ;;  (vm-set-debug-flags)
   ;; If this is the first time VM has been run in this Emacs session,
   ;; do some necessary preparations.
@@ -1172,6 +1554,25 @@ recipient list."
 	  (or (vm-load-window-configurations vm-window-configuration-file)
 	      (setq vm-window-configurations vm-default-window-configuration)))
 	(setq vm-buffers-needing-display-update (make-vector 29 0))
+	(if (if (fboundp 'find-face)
+		(find-face 'vm-monochrome-image)
+	      (facep 'vm-monochrome-image))
+	    nil
+	  (make-face 'vm-monochrome-image)
+	  (set-face-background 'vm-monochrome-image "white")
+	  (set-face-foreground 'vm-monochrome-image "black"))
+	(if (or (not vm-fsfemacs-p)
+		;; don't need this face under Emacs 21.
+		(fboundp 'image-type-available-p)
+		(facep 'vm-image-placeholder))
+	    nil
+	  (make-face 'vm-image-placeholder)
+	  (if (fboundp 'set-face-stipple)
+	      (set-face-stipple 'vm-image-placeholder
+				(list 16 16
+				      (concat "UU\377\377UU\377\377UU\377\377"
+					      "UU\377\377UU\377\377UU\377\377"
+					      "UU\377\377UU\377\377")))))
 	;; default value of vm-mime-button-face is 'gui-button-face
 	;; this face doesn't exist by default in FSF Emacs 19.34.
 	;; Create it and initialize it to something reasonable.
