@@ -113,11 +113,13 @@ all the real folder buffers involved."
 			 ;; set enable-local-variables to nil
 			 ;; for newer Emacses
 			 (let ((inhibit-local-variables t)
+			       (enable-local-eval nil)
 			       (enable-local-variables nil))
 			   (find-file-noselect folder)))))
 	    (set-buffer (or (and (bufferp folder) folder)
 			    (vm-get-file-buffer folder)
 			    (let ((inhibit-local-variables t)
+				  (enable-local-eval nil)
 				  (enable-local-variables nil))
 			      (find-file-noselect folder))))
 	    (if (eq major-mode 'vm-virtual-mode)
@@ -184,6 +186,8 @@ all the real folder buffers involved."
 		     message
 		     (vm-real-message-sym-of (car mp)))
 		    (vm-set-message-type-of message vm-folder-type)
+		    (vm-set-message-access-method-of
+		     message vm-folder-access-method)
 		    (vm-set-message-id-number-of message
 						 vm-message-id-number)
 		    (vm-increment vm-message-id-number)
@@ -260,7 +264,8 @@ Prefix arg means the new virtual folder should be visited read only."
   (vm-select-folder-buffer)
   (vm-check-for-killed-summary)
   (vm-error-if-folder-empty)
-  (let (vm-virtual-folder-alist)
+  (let ((use-marks (eq last-command 'vm-next-command-uses-marks))
+	vm-virtual-folder-alist)
     (if (null name)
 	(if arg
 	    (setq name (format "%s %s %s" (buffer-name) selector arg))
@@ -269,7 +274,10 @@ Prefix arg means the new virtual folder should be visited read only."
 	  (list
 	   (list name
 		 (list (list (list 'get-buffer (buffer-name)))
-		       (if arg (list selector arg) (list selector))))))
+		       (if use-marks
+			   (list 'and '(marked)
+				 (if arg (list selector arg) (list selector)))
+			 (if arg (list selector arg) (list selector)))))))
     (vm-visit-virtual-folder name read-only))
   ;; have to do this again here because the known virtual
   ;; folder menu is now hosed because we installed it while
@@ -294,12 +302,18 @@ Prefix arg means the new virtual folder should be visited read only."
   (vm-check-for-killed-summary)
   (vm-error-if-folder-empty)
   (let ((vfolder (assoc name vm-virtual-folder-alist))
+	(use-marks (eq last-command 'vm-next-command-uses-marks))
 	clauses vm-virtual-folder-alist)
     (or vfolder (error "No such virtual folder, %s" name))
     (setq vfolder (vm-copy vfolder))
     (setq clauses (cdr vfolder))
     (while clauses
       (setcar (car clauses) (list (list 'get-buffer (buffer-name))))
+      (if use-marks
+	  (setcdr (car clauses)
+		  (list (list 'and '(marked)
+			      (nconc (list 'or) (cdr (car clauses)))))))
+      (message "%S" (car clauses))
       (setq clauses (cdr clauses)))
     (setcar vfolder (format "%s/%s" (buffer-name) (car vfolder)))
     (setq vm-virtual-folder-alist (list vfolder))
@@ -323,7 +337,7 @@ Prefix arg means the new virtual folder should be visited read only."
 	      displayed-subject "\"\"")
       (setq subject (regexp-quote subject)))
     (vm-create-virtual-folder
-     'subject subject nil
+     'sortable-subject subject nil
      (format "%s %s %s" (buffer-name) 'subject displayed-subject))))
 
 (defun vm-create-virtual-folder-same-author ()
@@ -441,6 +455,9 @@ Prefix arg means the new virtual folder should be visited read only."
 (defun vm-vs-subject (m arg)
   (string-match arg (vm-su-subject m)))
 
+(defun vm-vs-sortable-subject (m arg)
+  (string-match arg (vm-so-sortable-subject m)))
+
 (defun vm-vs-sent-before (m arg)
   (string< (vm-so-sortable-datestring m) (vm-timezone-make-date-sortable arg)))
 
